﻿-- Macro Profiles
local MPS_Version = "2.0.4"

MPS = LibStub("AceAddon-3.0"):NewAddon("Profiles: Macros", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

local MAX_ACCOUNT_MACROS, MAX_CHARACTER_MACROS = 36, 18

--Char specific macros
minMacros = 1
charMacros = MAX_ACCOUNT_MACROS + 1
maxMacros = MAX_ACCOUNT_MACROS+MAX_CHARACTER_MACROS

function MPS:OnInitialize()
   MPdb = LibStub("AceDB-3.0"):New("ProfilesMacros",nil,"Restore [LOCKED]")
   MPglobaldb = LibStub("AceDB-3.0"):New("ProfilesMacrosG",nil,"Restore [LOCKED]")
end


function MPS:PLAYER_ENTERING_WORLD()
	MPS:Check_empty_macrotable()
end

-- If a totally new char has NO MACROS
-- then load RESTORE from the global DB
-- else save his macros in restore
function MPS:Check_empty_macrotable()
	local numperglobal,_ = GetNumMacros()
	
	-- Macro table empty, attempt to load Global Restore Profile
	if numperglobal == 0 and MPS:ExistsProfile("Restore [LOCKED]",1) then
		MPS:ImportGlobalProfile("Restore [LOCKED]")
		MPS:LoadMacroProfile("Restore [LOCKED]")
		return 1
		
	-- No global RESTORE, save	
	elseif not MPS:ExistsProfile("Restore [LOCKED]",1) and numperglobal > 0 then
		MPS:PreSaveMacroProfile("Restore [LOCKED]")
		MPS:ExportCharProfile("Restore [LOCKED]")
		return 2
	else
		return nil
	end
	
end

-- Export char profile to global
function MPS:ExportCharProfile(name)
	if MPS:ExistsProfile(name) then
		if MPS:ExistsProfile(name,1) then
			MPS:Echo("Overwritting global profile.."..name)
		end
		
		local orig_profile = MPdb:GetCurrentProfile()
		
		MPglobaldb:SetProfile(name)
		MPdb:SetProfile(name)
		
		-- MPglobaldb.profile = MPdb.profile
		for index, value in orderedPairs(MPdb.profile) do
		  MPglobaldb.profile[index] = value
		end
	
		MPdb:SetProfile(orig_profile)
		
		MPS:Echo("Profile exported, use import to load it: "..name)
		return 1
	else
		MPS:Echo("No such profile exists: "..name)
		return nil
	end
end

-- Import global profile to char
function MPS:ImportGlobalProfile(name)
	if MPS:ExistsProfile(name,1) then
		
		if MPS:ExistsProfile(name) then
			MPS:Echo("Overwritting profile.."..name)
		end
		
		-- TODO: SetProfile back to original
		MPdb:SetProfile(name)
		MPglobaldb:SetProfile(name)
		
		-- MPdb.profile = MPglobaldb.profile
		for index, value in orderedPairs(MPglobaldb.profile) do
		  MPdb.profile[index] = value
		end
		
		
		MPS:Echo("Profile imported: "..name)
		return 1
	else
		MPS:Echo("No such global profile exists: "..name)
		return nil
	end
end


-- TODO: Hook onto macrowindow to "lock" a macro
-- TODO: Hook onto macrowindow to select macros from a dropdown...ir if i should automate everything by spec.

function MPS:Echo(text)
	if PROFILES_TYST then return nil end
   DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000Profiles: Macros - "..text)
   return 1
end

-- ProfilesHub: Send message to all slaves that we are now saving profile
function MPS:PreSaveMacroProfile(name)
	MPS:SendAceMsg("save",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function MPS:SaveMacroProfile(name)
	
	-- Save a restore point if empty db
	if not MPS:ExistsProfile("Restore [LOCKED]") and not MPSSAVERESTORE then 
		MPSSAVERESTORE=1 
		MPS:SaveMacroProfile("Restore [LOCKED]") 
		MPSSAVERESTORE=nil
	end
	
	-- User trying to overwrite restore point!
	if name == "Restore [LOCKED]" and not MPSSAVERESTORE then 
		MPS:Echo("This profile is locked at the first run and can only be loaded!")
		return nil
	end
	
	-- No macros!
	local numperglobal,numperchar = GetNumMacros()
	if numperchar == 0 and numperglobal == 0 then 
		MPS:Echo("Nothing to save!") 
		return nil 
	end
	
	-- Duplicate macro name!
	-- Since we are using the name to identify where its actionBar_pos is, WARN user
	
	MPdb:SetProfile(name)
	
	-- Solving /mp save tank X 2
	if MPS:ExistsProfile(name) then MPdb:ResetProfile(name) end
	
	MPS:Echo("Saving macros to profile "..name.."!")
	
	
	-- Cache icons path
	local icons_table = {}
	local numIcons = GetNumMacroIcons()
	for i=1,numIcons do
		local lowercase_icons = string.lower(GetMacroIconInfo(i)) -- Normalize icon path from the API
		icons_table[lowercase_icons] = i
	end
    
	-- Save macros
	for i=minMacros,maxMacros do
		
		-- No more macros saved
		if GetMacroInfo(i) then
			local macro_name, texture, body = GetMacroInfo(i)
			local actionBar_pos = {};
		
			-- Get icon ID instead of path (needed for CreateMacro() in Load!
			texture = string.lower(texture)
			if icons_table[texture] then 
				texture = icons_table[texture]
			end
			  
			-- Icon not found, give question mark!
			if (type(texture) ~= "number") then 
				texture = '1'  
			end

			-- Save actionbar position
			for actionID=1, 120 do
				local _, id = GetActionInfo(actionID)
				local abmacro_name, _, _ = GetMacroInfo(id)

				if i == id then
					table.insert(actionBar_pos , actionID)
				end
			end
			
			-- Save the GLOBAL macro to db
			MPdb.profile[i] = { macro_name,texture,body,actionBar_pos,1,i}
		end
	end
		
	MPS:SendMessage("MPS_Profiles_save",name) -- ProfilesHub
	return name
end

-- This is an alias for ProfilesHub
function MPS:Save(...)
	MPS:SaveMacroProfile(...)
end


-- ProfilesHub: Send message to all slaves that we are now loading profile
function MPS:PreLoadMacroProfile(name)
	MPS:SendAceMsg("load",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function MPS:LoadMacroProfile(name)
	
	-- Save a restore point if empty db
	if not MPS:ExistsProfile("Restore [LOCKED]") and not MPSSAVERESTORE then 
		MPSSAVERESTORE=1 
		MPS:SaveMacroProfile("Restore [LOCKED]") 
		MPSSAVERESTORE=nil
	end
	
	-- No such profile saved
	if not MPS:ExistsProfile(name) then MPS:Echo(name.." profile not found.") return nil end
	
	-- In combat! 
	if InCombatLockdown() then MPS:Echo("Cannot change macros in combat!") return nil end

   MPdb:SetProfile(name)
  
   MPS:Echo("Loading macros from profile "..name.."!")
   
	--Get rid of all the current macros
	for i=minMacros,MAX_ACCOUNT_MACROS do DeleteMacro(1)  end -- Global
	for i=charMacros,maxMacros do DeleteMacro(37) end -- Char
	
   --Load saved macros profile
	for index, keys in pairs(MPdb.profile) do   
		if MPdb.profile[index] then
			if (type(keys[2]) ~= "number") then keys[2] = '1' end -- Icon not found, give question mark!
			
			if index >= charMacros then -- CreateMacro must know if its a global or char macro
				CreateMacro(keys[1], keys[2], keys[3], 1);
			else
				CreateMacro(keys[1], keys[2], keys[3], nil);
			end

		end
	end
   
	-- When having multiple name_duplicates the ID is changed..update the IDS
	for savedMacro, keys in pairs(MPdb.profile) do
		-- Find the loaded macro with body as ID!
		for loadedMacro=1,54 do
			if select(3,GetMacroInfo(loadedMacro)) then
				if keys[3] == select(3,GetMacroInfo(loadedMacro)) and keys[1] == select(1,GetMacroInfo(loadedMacro))  then -- if saved_body == loaded_body and saved_name == loaded_ame
				-- Update its macroID in the database
				
					if MPdb.profile[savedMacro][6] ~= loadedMacro then
						MPdb.profile[savedMacro][6] = loadedMacro
					end
				end
			end
		end
	end
	
   
   
   -- Action bar handling
   for index, keys in pairs(MPdb.profile) do   
       if MPdb.profile[index] and keys[4] then	
			for _,v in pairs(keys[4]) do
				ClearCursor()
				
				-- Clear current spot
				PickupAction(v)
				ClearCursor()

				-- Restore this spot
				PickupMacro(keys[6])
				PlaceAction(v)
				ClearCursor()
			end
			
       end
   end
   
	if (IsAddOnLoaded("Blizzard_MacroUI")) then
		MacroFrame_Update() -- Blizzards own code
	end
   
	MPS:SendMessage("MPS_Profiles_load",name) -- ProfilesHub
	return name
end

-- This is an alias for ProfilesHub
function MPS:Load(...)
	MPS:LoadMacroProfile(...)
end

-- db = 1 -- global database
function MPS:ExistsProfile(name,db)
	
	if db then 
		for i, k in ipairs(MPglobaldb:GetProfiles()) do
			if k == name then
				return name
			end
		end
	else
		for i, k in ipairs(MPdb:GetProfiles()) do
		
			if k == name then
				return name
			end
		end
	end
   

	return nil
end

function MPS:ListProfiles()
    MPS:Echo(" Available Profiles: ");
    
	for i,k in ipairs(MPdb:GetProfiles()) do
		if k == MPdb:GetCurrentProfile() then
			MPS:Echo(' ' .. k.." (current)")
		else
			MPS:Echo(' ' .. k)
		end
	end
	
	return 1
end

-- ProfilesHub: Send message to all slaves that we are now deleting a profile
function MPS:PreDeleteProfile(name)
	MPS:SendAceMsg("delete",name)
end

function MPS:DeleteProfil(name)
	if name == "Restore [LOCKED]" and not MPSSAVERESTORE then 
		MPS:Echo("This profile cannot be deleted! It is locked at the first run and can only be loaded!")
		return nil
	end
	
	if (MPS:ExistsProfile(name)==nil)then
		MPS:Echo(name.." doesnt exist.")
		MPS:ListProfiles()
		
		return nil
	else
		if name == MPdb:GetCurrentProfile() then
			MPdb:SetProfile('Default')
		end
		MPdb:DeleteProfile(name)
		MPS:Echo(name.." deleted.")
		
		return name
	end
end

-- This is an alias for ProfilesHub
function MPS:Delete(...)
	MPS:DeleteProfil(...)
end